; File:         print.inc
; Author:       RuiLoureiro
; Date:         14 /04 /2005
; 

; 
; Action:   Opens the window defined by the pointer pWnd
;
; Input:
;       pWnd    - buffer pointer 
;
; Output:
;       clc: OK
;
;       stc: error
;
; Info:
;
;
WindowOpen          proc pWnd:DWORD
                    push    ebx 
                    ;
                    mov     ebx, pWnd
                    call    WindowRead
                    jc      short @F
                    ;
                    ; Draw new window EBX
                    ; -------------------
                    call    WindowMake
                    ;
@@:                 pop     ebx
                    ret
WindowOpen          endp
; ---------------------------------------------------------------------------------------------
; Action:   Closes the window defined by the pointer pWnd
;
; Input:
;       pWnd    - window pointer 
;
; Output:
;       clc: OK
;
;       stc: error
;
; Info:
;
;
WindowClose         proc  pWnd:DWORD
                    push  ebx
                    ;
                    mov    ebx, pWnd
                    call   WindowWrite
                    ;
                    pop    ebx
                    ret
WindowClose         endp
; ---------------------------------------------------------------------------------------------
;
;
; Input:
;       ebx    - buffer pointer 
;
; Output:
;       clc: OK
;
;       stc: error
;
; Info:
;
;
WindowRead          proc
                    LOCAL   pWnd:DWORD
                    pushad 
                    ;
                    mov     pWnd, ebx
                    cmp     dword ptr [ebx - WndFlgOpn$], 0
                    je      short @F
                    ;
                    ; is full
                    ;
                    stc
                    popad
                    mov     eax, 1
                    ret
                    ;
@@:                 call   WindowDefRect
                    jc     short _eWindowRead
                    ;
                    xor     edx, edx
                    invoke   ReadConsoleOutput, _hScreenBuffer,
                                                ebx,
                                                ecx,
                                                edx,
                                                ADDR _CoordWrite
                    cmp    eax, 0
                    jne    short @F
                    ;
                    stc
                    popad
                    mov     eax, 0
                    ret
                    ;
@@:                 mov     ebx, pWnd
                    mov     dword ptr [ebx - WndFlgOpn$], 1
_eWindowRead:       clc
                    popad
                    ret
WindowRead          endp
; ---------------------------------------------------------------------------------------------
;
; Input:
;       ebx    - window pointer 
;
WindowWrite         proc
                    LOCAL   pWnd:DWORD
                    pushad
                    ;
                    ;
                    ;
                    mov     pWnd, ebx
                    cmp     dword ptr [ebx - WndFlgOpn$], 1
                    je      short @F
                    ;
                    stc
                    popad
                    ret
                    ;
                    ; Get from the structure
                    ; ----------------------
@@:                 call   WindowDefRect
                    ;
                    xor     edx, edx
                    invoke  WriteConsoleOutput, _hScreenBuffer,
                                                ebx,                ; buffer pointer
                                                ecx,                ; number Lin-Col
                                                edx,                ; ini buffer
                                                ADDR _CoordWrite
                    cmp    eax, 0
                    jne    short @F
                    ;
                    stc
                    popad
                    mov     eax, 0
                    ret
                    ;
@@:                 mov     ebx, pWnd
                    mov     dword ptr [ebx - WndFlgOpn$], 0
                    clc
                    ;
                    popad
                    ret
WindowWrite         endp
; ---------------------------------------------------------------------------------------------
; Output:
;        clc:
;           rectangle defined in _CoordWrite          
;           ECX - number of Lin + Col = buffer size
;
;        stc:  buffer error
;
WindowDefRect       proc    
                    ;
                    mov     eax, dword ptr [ebx - WndHeight$]   ; height
                    mov     ecx, dword ptr [ebx - WndWidth$]    ; width
                    mul     ecx
                    cmp     eax, dword ptr [ebx - WndLenBuf$]
                    jbe     short @F
                    stc
                    ret
                    ;
@@:                 mov     eax, dword ptr [ebx - WndLinOpn$]   ; line
                    dec     eax
                    mov     word ptr [_CoordWrite + 2], ax                  ; top
                    ;
                    mov     edx, dword ptr [ebx - WndColOpn$]   ; column
                    dec     edx
                    mov     word ptr [_CoordWrite + 0], dx                  ; left
                    ;
                    mov     ecx, dword ptr [ebx - WndWidth$]    ; width
                    add     dx, cx
                    dec     dx
                    mov     word ptr [_CoordWrite + 4], dx                  ; right
                    mov     edx, ecx
                    ;
                    mov     ecx, dword ptr [ebx - WndHeight$]   ; height
                    add     ax, cx
                    dec     ax
                    mov     word ptr [_CoordWrite + 6], ax                  ; bottom
                    ;
                    shl     ecx, 16
                    mov     cx, dx  
                    clc    
                    ret
WindowDefRect       endp
; 
; Action:
;        Draw the window in the buffer _WindowBuffer and put it in the screen buffer
;
; Input:
;       ebx    - window pointer 
;
; Output:
;        clc:  OK
;
;        stc:  error
;
WindowMake          proc    
                    pushad
                    ;
                    call    WindowGetpType
                    jc      short _eWindowMake
                    ;
                    mov     eax, dword ptr [ebx - WndAtrOnp$]   ; color
                    shl     eax, 16
                    ;
                    mov     edx, dword ptr [ebx - WndWidth$]    ; width
                    mov     ecx, dword ptr [ebx - WndHeight$]   ; height
                    sub     ecx, 2
                    sub     edx, 2
                    ;
                    ;
                    ;
                    mov     edi, offset _WindowBuffer
                    call    LineSupDraw
                    ;
@@:                 call    LineMidDraw
                    loop    short @B
                    ;                 
                    call    LineInfDraw
                    ; ---------
                    ; Add Title
                    ; ---------
                    mov     edi, offset _WindowBuffer
                    call    TitleAdd
                    ; --------------
                    ; Add Msg or Tbl
                    ; --------------
                    call    WindowMsgAdd
                    ;
                    ; transfer to screen
                    ; ------------------
@@:                 mov     ecx, dword ptr [ebx - WndHeight$]   ; height
                    shl     ecx, 16
                    mov     eax, dword ptr [ebx - WndWidth$]    ; width
                    mov     cx, ax      
                    call    WindowPutScreen

                    ; ---------------------
                    ; Prc table of messages
                    ; ---------------------
                    mov     eax, dword ptr [ebx - WndMsgPrc$]
                    or      eax, eax
                    jz      short _eWindowMake
                    ;
                    call     dword ptr [ebx - WndMsgPrc$]      ; it works 
                    ;
_eWindowMake:       popad
                    ret
WindowMake          endp
; ---------------------------------------------------------------------------------------------
;
;
; Input:
;       ebx    - window pointer 
;
TitleAdd            proc   
                    pushad
                    ;
                    mov     esi, dword ptr [ebx - WndpTitle$]   ; pTitle
                    or      esi, esi
                    jz      short _eTitleAdd
                    ;
                    cld
                    mov     ecx, dword ptr [esi - $PasLen$]
                    mov     eax, dword ptr [ebx - WndaTitle$]            
                    shl     eax, 16
                    ;            
                    mov     edx, dword ptr [ebx - WndWidth$]    ; width
                    cmp     ecx, edx
                    jb      short @F
                    je      short _TitleAdd1
                    ;
                    mov     ecx, edx
                    jne     short _TitleAdd1
                    ;
@@:                 sub     edx, ecx
                    shr     edx, 1
                    ;
                    shl     edx, 2
                    add     edi, edx
                    ;
_TitleAdd1:         lodsb
                    stosd
                    loop   short _TitleAdd1

_eTitleAdd:         popad
                    ret
TitleAdd            endp
; ---------------------------------------------------------------------------------------------
; Input:
;       ebx    - window pointer 
;
WindowMsgAdd        proc   
                    pushad
                    ;
                    mov     esi, dword ptr [ebx - WndpMsg$]     ; pMsg
                    or      esi, esi
                    jz      _eWindowMsgAdd                      ; hasnt Msg
                    ;
                    ; Type ?
                    ;
                    mov     ecx, dword ptr [esi - $PasLen$]     ; Msg len
                    mov     eax, dword ptr [ebx - WndTypMsg$]
                    cmp     eax, $PasStr$                       ; string PAS
                    je      _WindowMsgAdd3
                    ;
                    mov     ecx, dword ptr [esi - $RclLen$]     ; Msg len
                    cmp     eax, $RclStr$                       ; string RCL
                    je      _WindowMsgAdd3
                    ;
                    ; tables Pas / Rcl
                    ; ----------------
                    mov     ecx, dword ptr [esi - $HTbl$]       ; 
                    xor     edx, edx
                    ;
                    cmp     eax, $PasTbl$                       ; Pascal table
                    je      short _WindowMsgAdd2
                    ;
                    cmp     eax, $RclTbl$                       ; RCL table
                    jne     _eWindowMsgAdd                      ; error
                    ;
                    ; Tbl Rcl
                    ; -------
                    call    WindowMovTbl
                    ;
                    popad
                    ret
;...........................................................
                    ;
                    ; Tbl PAS
                    ; -------
_WindowMsgAdd2:     call    WindowMovTbl
                    ;
                    popad
                    ret
;...........................................................
                    ;
                    ; Pas / Rcl
                    ; ---------
_WindowMsgAdd3:     call    MsgDefAddr
                    jc      short _eWindowMsgAdd
                    ;
                    mov     eax, dword ptr [ebx - WndAtrMsg$]  ; Msg colour            
                    shl     eax, 16
                    cld
                    ;
@@:                 lodsb
                    stosd
                    loop    short @B
                    ;
_eWindowMsgAdd:     popad
                    ret
WindowMsgAdd        endp
; ---------------------------------------------------------------------------------------------
; Input:
;        eax= $PasTbl$  => Pascal table
;        eax= $RclTbl$  => RCL    table        
;        ebx   -
;        esi   -
;        edi   -
;
WindowMovTbl        proc
                    LOCAL Col:DWORD
                    LOCAL TblType:DWORD
                    ;
                    mov     dword ptr TblType, eax
                    ;
_WindowMovTbl0:     push    edx
                    push    ecx
                    push    esi
                    push    edi
                    ;
                        shl      edx, 4
                        add      esi, edx
                        ;
                        lodsd
                        mov      ecx, eax
                        lodsd
                        mov      edx, eax                         ; lin
                        lodsd
;                        mov      ebp, eax                         ; col
                        mov      dword ptr Col, eax
                        lodsd
                        shl      eax, 16
                        ;
                        mov     esi, ecx
                        mov     ecx, dword ptr [esi - $PasLen$]    ; Msg len
                        ;
                        cmp     TblType, $PasTbl$            ; Pascal table
                        je      short @F
                        ;
                        mov     ecx, dword ptr [esi - $RclLen$]    ; Msg len
                        ;
@@:                     call    MsgDefPosLin
                        jc      short _WindowMovTbl1
                        ;
                        ;
                        ;
                        ;mov     edx, ebp                           ; col Msg
                        mov     edx, dword ptr Col                 ; col Msg
                        call    MsgDefPosCol
                        jc      short _WindowMovTbl1
                        ;
                        cld
@@:                     lodsb
                        stosd
                        loop    short @B
                    ;
_WindowMovTbl1:     pop     edi
                    pop     esi
                    pop     ecx
                    pop     edx
                    ;
                    inc     edx
                    loop    short _WindowMovTbl0
                    ;
                    ret
WindowMovTbl        endp
; ---------------------------------------------------------------------------------------------
; Input:
;        EBX   - window pointer
;        ESI   - pPas / pRcl pointer
;        EBP   - len 
;
; Output:
;        EDI   - new address where to write
;
MsgDefAddr          proc
                    push    ecx
                    ;
                    mov     edx, dword ptr [ebx - WndLinMsg$]   ; Msg line
                    call    MsgDefPosLin
                    jc      short _eMsgDefAddr
                    ;
                    ;
                    ;
                    cmp     dword ptr [ebx - WndJstMsg$], 0
                    je      short @F
                    ;
                    ; Centre
                    ; ------
                    mov     edx, dword ptr [ebx - WndWidth$]   ; width
                    sub     edx, 2
                    ;
                    cmp     ecx, edx
                    jae     short _MsgDefAddr1 
                    ;
                    sub     edx, ecx
                    shr     edx, 1
                    shl     edx, 2
                    ;
                    add     edi, edx
                    jmp     _MsgDefAddr1
                    ;
@@:                 mov     edx, dword ptr [ebx - WndColMsg$]   ; col Msg
                    call    MsgDefPosCol
                    jc      short _eMsgDefAddr
                    ; 
_MsgDefAddr1:       clc
                    pop    ecx
                    ret
                    ;
_eMsgDefAddr:       stc
                    pop    ecx
                    ret
MsgDefAddr          endp  
; ---------------------------------------------------------------------------------------------
; Input:
;        EBX   - window pointer
;        EDX   - Msg line
;        EDI   - buffer pointer
;
; Output:
;        EDI   - new pos
;
;
; Info:
;        Preserves all other registers
;
MsgDefPosLin        proc
                    push   eax 
                    push   ecx
                    push   edx
                    ;
                    mov     ecx, dword ptr [ebx - WndHeight$]   ; height
                    mov     eax, dword ptr [ebx - WndLinOpn$]   ; Opn line
                    cmp     edx, eax           
                    jbe     short @F
                    ;
                    sub     edx, eax
                    ;
                    mov     eax, dword ptr [ebx - WndWidth$]    ; width
                    cmp     edx, ecx           
                    jae     short @F
                    ;
                    ;
                    ;
                    mul     edx
                    shl     eax, 2
                    add     edi, eax
                    ;
                    clc
                    pop     edx
                    pop     ecx 
                    pop     eax
                    ret
                    ;
@@:                 pop     edx
                    pop     ecx 
                    pop     eax
                    stc
                    ret
MsgDefPosLin        endp 
; ---------------------------------------------------------------------------------------------
; Input:
;        EBX   - window pointer
;        EDX   - Msg line
;        EDI   - buffer pointer
;
; Output:
;        EDI   - new pos
;
; Info:
;        Preserves all other registers
;
MsgDefPosCol        proc
                    push    eax
                    push    edx
                    ;
                    mov     eax, dword ptr [ebx - WndColOpn$]   ; Msg col
                    cmp     edx, eax
                    jbe     short @F
                    ;
                    sub     edx, eax
                    shl     edx, 2
                    add     edi, edx
                    ;
                    clc
                    pop     edx
                    pop     eax
                    ret
                    ;
@@:                 stc
                    pop     edx
                    pop     eax
                    ret 
MsgDefPosCol        endp                 
; ---------------------------------------------------------------------------------------------
; Action:
;        
;
WindowSetPrc        proc Wnd:DWORD, pPrc:DWORD
                    push   ebx
                    ;
                    mov    ebx, Wnd
                    mov    eax, pPrc
                    mov    dword ptr [ebx - WndMsgPrc$], eax
                    ;
                    pop    ebx
                    ret
WindowSetPrc        endp
; ---------------------------------------------------------------------------------------------
; Input:
;      Wnd     - window pointer
;      pTbl    - table pointer
;      tTbl    - 0 = table with Pascal strings, 1= Rcl table with Rcl strings
;
WindowSetTbl        proc Wnd:DWORD, pTbl:DWORD, tTbl:DWORD
                    push   ebx
                    ;
                    mov    ebx, Wnd
                    mov    eax, pTbl
                    mov    dword ptr [ebx - WndpMsg$], eax
                    mov    eax, tTbl
                    add    eax, 2
                    mov    dword ptr [ebx - WndTypMsg$], eax
                    ;
                    pop    ebx
                    ret
WindowSetTbl        endp
; ---------------------------------------------------------------------------------------------
WindowGetpType      proc    
                    push    eax
                    ;
                    mov     eax, dword ptr [ebx - WndType$]      ; type
                    mov     esi, offset _JanX
                    cmp     eax, 1
                    je      short @F
                    ;
                    mov     esi, offset _JanY
                    cmp     eax, 2
                    je      short @F
                    ;
                    mov     esi, offset _JanW
                    cmp     eax, 3
                    je      short @F
                    ;
                    mov     esi, offset _JanV
                    cmp     eax, 4
                    je      short @F
                    ;
                    mov     esi, offset _JanT
                    cmp     eax, 5
                    je      short @F
                    ;
                    stc
                    pop     eax
                    ret
                    ;
@@:                 clc
                    pop     eax
                    ret
WindowGetpType      endp
; ---------------------------------------------------------------------------------------------
;
; Input:
;       ecx    - number of lines and columns
;
WindowPutScreen     proc    
                    pushad
                    ;
                    mov     ebx, offset _WindowBuffer
                    xor     edx, edx
                    invoke  WriteConsoleOutput, _hScreenBuffer,
                                                ebx,                ; buffer pointer
                                                ecx,                ; number Lin-Col
                                                edx,                ; ini buffer
                                                ADDR _CoordWrite
                    cmp    eax, 0
                    jne    short @F
                    ;
                    stc
                    popad
                    mov     eax, 0
                    ret

@@:                 clc
                    popad
                    ret
WindowPutScreen     endp
; 
; Input:
;        EDX    - width - 2
;	   ECX    - height - 2
;        EAX    - colour
;        ESI    - 
;        EDI    - buffer where draw
;
LineSupDraw		  proc
                    push    eax
                    push    ecx	
                    ;
                    cld
			  mov	   al, byte ptr [esi + CantoSupEsq$]
			  stosd
			  ;
			  mov	   al, byte ptr [esi + LinhaHoriz$]
			  mov	   ecx, edx
			  rep	   stosd
			  ;
			  mov	   al, byte ptr [esi + CantoSupDir$]
			  stosd
			  ;
                    pop     ecx
                    pop     eax
			  ret
LineSupDraw		  endp
; ---------------------------------------------------------------------------------------------
; Input:
;        EDX    - width - 2
;	   ECX    - height - 2
;        EAX    - colour
;        ESI    - 
;        EDI    - buffer where draw
;
LineMidDraw		  proc	
                    push    eax
                    push    ecx	
                    ;
			  cld
			  ;
			  ; Linhas intermdias
			  ; ------------------
			  mov	   al, byte ptr [esi + LinhaVert$]
			  stosd
			  ;
			  mov	   al, 32
			  mov	   ecx, edx
			  rep	   stosd
			  ;
			  mov	   al, byte ptr [esi + LinhaVert$]
			  stosd
			  ;
                    pop     ecx
                    pop     eax
			  ret
LineMidDraw		  endp
; ---------------------------------------------------------------------------------------------
; Input:
;        EDX    - width - 2
;	   ECX    - height - 2
;        EAX    - colour
;        ESI    - 
;        EDI    - buffer where draw
;
LineInfDraw		  proc
                    push    eax
                    push    ecx	
                    ;
			  cld
			  mov	   al, byte ptr [esi + CantoInfEsq$]
			  stosd
			  ;
			  mov	   al, byte ptr [esi + LinhaHoriz$]
			  mov	   ecx, edx
			  rep	   stosd
			  ;
			  mov	   al, byte ptr [esi + CantoInfDir$]
			  stosd
			  ;
                    pop     ecx
                    pop     eax
			  ret
LineInfDraw         endp
; 
;                                       INFO
; ---------------------------------------------------------------------------------------------
; Action:
;        Gets Screen Buffer info
;
; Input:
;       _hScreenBuffer  - screen buffer handle
;       _ScreenInfo     - screen info   variable
;
; Output:
;       _ScreenInfo     - screen info   values
;       _NewWindow      - max window position       (left, top, right, bottom)
;       _MaxWinCOLLIN   - max window COL (low word) + LIN (high word)
;       _MaxBufferSize  - max buffer size
;       _CursorPosition - cursor position
;       _ColorAttr      - color attribute
;
; Info:
;
;
InfoGet         proc
                
                ; ----------------------------------
                ; Get largest window screen buffer
                ; ----------------------------------
                invoke   GetLargestConsoleWindowSize, _hScreenBuffer
                mov      dword ptr _MaxWinCOLLIN, eax                   ; low= COL high= LIN    

                mov     _MaxBufferSize.x, ax

                ; ----------------------
                ; Get screen buffer info
                ; ----------------------
                invoke   ScreenInfoGet, _hScreenBuffer
                ;
                dec     ax
                dec     bx
                ;
                mov     _NewWindow.Left, 0
                mov     _NewWindow.Top,  0
                mov     _NewWindow.Right, ax
                mov     _NewWindow.Bottom, bx

                mov     _MaxBufferSize.y, cx

                mov     eax, _ScreenInfo.dwCursorPosition
                mov     _CursorPosition. x, ax
                mov     _CursorPosition. y, bx
                
                mov     bx,  _ScreenInfo.wAttributes
                mov     _ColorAttr, bx

                movzx   eax, _ScreenInfo.srWindow.Left
                movzx   ebx, _ScreenInfo.srWindow.Top
                movzx   ecx, _ScreenInfo.srWindow.Right
                movzx   edx, _ScreenInfo.srWindow.Bottom
                ; ---------------
                ; Get Cursor info
                ; ---------------
                invoke  GetConsoleCursorInfo, _hScreenBuffer,
                                             ADDR _CursorInfo
                ret
InfoGet         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Sets New Screen Buffer dimention
;
; Input:
;       _hScreenBuffer  - screen buffer handle
;
; Output:
;       nothing
;
; Info:
;
WindowSetDim        proc nLin:DWORD, nCol:DWORD    
                    pushad
                    ;
                    mov     eax, nLin
                    shl     eax, 16

                    mov     edx, nCol
                    mov     ax, dx
                    mov     _MaxBufferSize, eax
                    ;
                    invoke SetConsoleScreenBufferSize, _hScreenBuffer, eax
                    ;
                    popad
                    ret
WindowSetDim        endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Sets New Screen Buffer Size
;
; Input:
;       _hScreenBuffer  - screen buffer handle
;       NewBufferSize   - new buffer size
;
; Output:
;       nothing
;
; Info:
;
ScreenBufferSet     proc    NewBufferSize:DWORD
                    invoke SetConsoleScreenBufferSize, _hScreenBuffer, 
                                                        NewBufferSize   ; max buffer size
                    ret
ScreenBufferSet     endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Cleans the screen
;
; Input:
;        hScr  - Screen buffer handle
;        Char  - ascii code which with it cleans
;        Atr   - colour attribute
;
; Output:
;        nothing
;
; Info:
;        preserves all registers
;
ScreenClear         proc    hScr:DWORD, Char:DWORD, Atr:DWORD
                    LOCAL   wNum:DWORD
                    pushad
                    ;
                    invoke    ScreenInfoGet, hScr                   ; Gets Screen info

                    mov    eax, ecx                   ; height
                    mul    edx                        ; width
                    mov    ecx, eax
                    ;
                    push   eax                                      ; we dont trust !
                    ;
                    mov    edx, Atr
                    invoke FillConsoleOutputAttribute, hScr, 
                                                       edx,         ; attribute
                                                       ecx,         ; number to write
                                                       0,           ; first cell
                                                       ADDR wNum    ; number written
                    pop    eax
                    ;
                    mov    edx, Char
                    invoke FillConsoleOutputCharacter, hScr, 
                                                       edx,         ; Char
                                                       ecx,         ; number to write
                                                       0,           ; first cell
                                                       ADDR wNum    ; number written
                    ;
                    popad
                    ret
ScreenClear         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Gets the Screen Buffer info to _ScreenInfo variable
;
; Input:
;       _ScreenInfo     - screen info variable
;
; Output:
;       _ScreenInfo  - screen info values
;               edx  - width
;               ecx  - height
;               eax  - MAX width
;               ebx  - MAX height
; Info:
;
;
ScreenInfoGet   proc hScr:DWORD
                
                ; ----------------------
                ; Get screen buffer info
                ; ----------------------
                invoke   GetConsoleScreenBufferInfo, hScr,
                                                    ADDR _ScreenInfo

                movzx   edx, _ScreenInfo.dwSize.x                       ; width
                movzx   ecx, _ScreenInfo.dwSize.y                       ; height
                ;
                movzx   eax,  _ScreenInfo.dwMaximumWindowSize.x
                movzx   ebx,  _ScreenInfo.dwMaximumWindowSize.y
                ;
                ret
ScreenInfoGet   endp
; 

