; File:         rcl.inc
; Author:       RuiLoureiro
; Date:         4 /Set /2001
; 
; p   means pointer;  l  means length
; RCL means strings of the type:         
;                                           dd Len              ; - 8
;                                           dd Max              ; - 4
;                               addressRCL  db Max dup (?)
; PAS means strings of the type:         
;                                           dd Max=Len          ; - 4
;                               addressPAS  db Max dup (?)
;...............................................................................
; In RCL strings, whenever possible:
;                EBX    - initial address
;                ECX    - length = number of bytes in the buffer
;                EAX    - max number of bytes
;                EDX    - current position
;...............................................................................
; Action:
;        Print the RCL string pointed by EBX
;
; Input:
;       EBX     - RCL string pointer
;       nLin    - Line
;       nCol    - Column
;       Atr     - colour attribute
;
; Output:
;        clc:  Ok
;              
;        stc:  system error 
;
; Info:
;       _COLLINCur    = current cursor location
;       _hScreenBuffer = screen buffer handle
;
RCLPrintLen         proc    nLin:DWORD, nCol:DWORD, Atr:DWORD
                    pushad
                    ;
                    mov     esi, ebx
                    mov     edx, nLin
                    mov     ebx, nCol
                    mov     eax, Atr
                    call    RCLPrint
                    ;
                    popad
                    ret
RCLPrintLen         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Print the RCL string pointed by EBX
;
; Input:
;       EBX     - RCL string pointer
;       nLin    - initial line
;       nCol    - initial column
;       Atr     - colour attribute
;
; Info:
;       _COLLINCur     = current cursor location
;       _hScreenBuffer = screen buffer handle
;
RCLCleanMax         proc    nLin:DWORD, nCol:DWORD, Atr:DWORD
                    pushad
                    ;
                    mov     ecx, dword ptr [ebx - $RclMax$]
                    ;
                    ; Set _AuxBuffer with Len chars 20h
                    ; ---------------------------------
                    mov     edi, offset _AuxBuffer
                    cmp     ecx, dword ptr [edi - $RclMax$]
                    jbe     short @F
                    ;
                    mov     ecx, dword ptr [edi - $RclMax$]
                    ;
@@:                 mov     dword ptr [edi - $RclLen$], ecx
                    ;
                    mov     al, byte ptr _CleanChar
                    cld
                    rep     stosb
                    ;
                    mov     ebx, offset _AuxBuffer 
                    invoke  RCLPrintLen, nLin, nCol, Atr
                    ;
_eRCLCleanMax:      popad
                    ret
RCLCleanMax         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Print the RCL string pointed by EBX
;
; Input:
;       ESI    - RCL string pointer
;       EDX    - Line
;       EBX    - Column
;       EAX    - colour attribute
;
; Output:
;        clc:  Ok
;              
;        stc:  system error 
;
; Info:
;       _COLLINCur    = current cursor location
;       _hScreenBuffer = screen buffer handle
;
RCLPrint            proc    
                    push   ecx
                    ;
                    mov     ecx, dword ptr [esi - $RclLen$]
                    call    BufferPrint
                    ;
                    pop    ecx
                    ret
RCLPrint            endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Prints the table of RCL strings. The table has (pPas, nLin, nCol, Atr)
;
; Input:
;        pTbl    - Table of Pascal strings
;
; Info:
;       _COLLINCur     = current cursor location
;       _hScreenBuffer = screen buffer handle
;
RCLPrintTbl         proc  pTbl:DWORD
                    pushad
                    ;
                    mov    esi, pTbl
                    mov    ecx, dword ptr [esi - $HTbl$]
                    or     ecx, ecx
                    jz     short _eRCLPrintTbl
                    ;
                    xor    ebx, ebx
                    ;
_RCLPrintTbl0:      push   ebx 
                    push   ecx 
                    push   esi
                        ;
                        cld
                        shl   ebx, 4
                        add   esi, ebx
                        ;
                        lodsd
                        mov   ecx, eax             ; pRCL
                        lodsd
                        mov   edx, eax             ; Lin
                        lodsd
                        mov   ebx, eax             ; Col
                        lodsd                      ; Colour
                        ;
                        mov     esi, ecx
                        call    RCLPrint
                        ;
                    pop    esi 
                    pop    ecx 
                    pop    ebx
                    ;
_RCLPrintTbl1:      inc    ebx
                    loop   short _RCLPrintTbl0
                    ;
_eRCLPrintTbl:      popad
                    ret
RCLPrintTbl         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Print the RCL string pointed by EBX
;
; Input:
;        EBX    - RCL string pointer
;        nLin   - initial line
;        nCol   - initial column
;
; Info:
;       _COLLINCur     = current cursor location
;       _hScreenBuffer = screen buffer handle
;
RCLClnPrint         proc    nLin:DWORD, nCol:DWORD, Atr:DWORD
                    ;
                    invoke   RCLCleanMax, nLin, nCol, Atr
                    invoke   RCLPrintLen, nLin, nCol, Atr
                    ret
RCLClnPrint         endp
;----------------------------------------------------------------------------------------------
; Action:
;        Print all RCL string pointed by pRCLStr
;
; Input:
;       EBX     - RCL string pointer
;       nLin    -
;       nCol    -
;
; Info:
;       _COLLINCur     = current cursor location
;       _hScreenBuffer = screen buffer handle
;
RCLPrintAll         proc    nLin:DWORD, nCol:DWORD, Atr:DWORD
                    LOCAL   wRCLStr:DWORD
                    push    ebx
                    ;
                    invoke   CursorColorSet, nLin, nCol, Atr
                    ;
                    mov     ecx, dword ptr [ebx - $RclMax$]
                    cmp     ecx, 0
                    je      _eRCLPrintAll
                    ;
                    invoke  WriteConsole, _hScreenBuffer,       ; handle
                                          ebx,                  ; print buffer
                                          ecx,                  ; write ECX char
                                          ADDR wRCLStr,         ; Number written
                                          0                     ;
_eRCLPrintAll:      pop ebx
                    ret
RCLPrintAll         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Add byte given in AL to RCL string pointed by EBX
;
; Input:
;       AL      - byte
;       EBX     - RCL string pointer
;
; Info:
;       clc:    byte added
;       stc:    string is full
;
RCLAddBytAL         proc  
                    push    ecx
                    ;
                    mov     ecx, dword ptr [ebx - $RclLen$]        ; Len
                    cmp     ecx, dword ptr [ebx - $RclMax$]        ; Max
                    jb      short @F
                    ;
                    stc
                    popad
                    ret
                    ;
@@:                 mov     byte ptr [ebx + ecx], al
                    ;
                    inc     ecx
                    mov     dword ptr [ebx - $RclLen$], ecx        ; Len
                    clc
                    ;
                    pop     ecx
                    ret
RCLAddBytAL         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Add byte given in AL to RCL string pointed by EBX
;
; Input:
;       AL      - byte
;       EBX     - RCL string pointer
;       lMax    - len max
;
; Info:
;       clc:    byte added
;       stc:    string is full
;
RCLAddFulAL         proc       
                    pushad
                    ;
                    mov     ecx, dword ptr [ebx - $RclLen$]        ; Len
                    ;
                    cmp     edx, dword ptr [ebx - $RclMax$]        ; Max
                    jbe     short @F
                    ;
                    mov     edx, dword ptr [ebx - $RclMax$]        ; Max
                    ;
@@:                 cmp     ecx, edx
                    jb      short @F
                    ;
                    stc
                    popad
                    ret
                    ;
@@:                 mov     edi, ebx
                    add     edi, ecx
                    ;
                    xchg    ecx, edx
                    sub     ecx, edx
                    ;
                    cld
                    rep     stosb
                    clc
                    ;
                    popad
                    ret
RCLAddFulAL         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Sets the byte given in AL to RCL string pointed by EBX at the position EDX.
;
; Input:
;       AL      - byte
;       AH      - background char  
;       EBX     - RCL string pointer
;       EDX     - position (0,1,2,..., Max-1)
;
; Info:
;       clc:    Len was altered    
;       stc:     "  not    "
;
RCLPutBytAL         proc    
                    pushad
                    ;
                    mov     byte ptr [ebx + edx], al
                    call    RCLGetLen
                    jb      short _RCLPutBytAL2
                    ; ---------
                    ; Len = Max
                    ; ---------
                    dec     ecx
                    cmp     edx, ecx
                    jb      short _eRCLPutBytAL
                    ;
                    ;
                    ;
_RCLPutBytAL1:      cmp     al, ah
                    jne     short _eRCLPutBytAL
                    ;
                    mov     dword ptr [ebx - 8], ecx          ; Len = Len - 1
                    je      short _RCLPutBytAL4

                    ; --------- 
                    ; Len < Max
                    ; ---------
_RCLPutBytAL2:      cmp     ecx, 0
                    je      short _RCLPutBytAL3 
                    ;
                    dec     ecx
                    cmp     edx, ecx
                    je      short _RCLPutBytAL1
                    jb      short _eRCLPutBytAL 
                    ;
                    ; 
                    ;
_RCLPutBytAL3:      cmp     al, ah          
                    je      short _eRCLPutBytAL 
                    ;
                    inc     edx
                    mov     dword ptr [ebx - $RclLen$], edx             ; Len = edx + 1
                    ;
_RCLPutBytAL4:      call    RCLDetNewLen
                    ;
                    clc
                    popad
                    ret
                    ;
_eRCLPutBytAL:      stc
                    popad
                    ret
RCLPutBytAL         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Shift left the string pointed by EBX to the position EDX
;
; Input:
;       ebx     - RCL string pointer
;       edx     - position (0,1,2,..., Max-1)
;       ah      - background char  
;
; Info:
;       clc:    string shifted
;       stc:    Len <> Max and Len < edx 
;
RCLShlPos           proc    
                    pushad
                    ;
                    call    RCLGetLen
                    jb      short _RCLShlPos1
                    ;
                    ; full
                    ; ----
                    dec     ecx
                    add     edi, edx
                    sub     ecx, edx
                    jz      short _RCLShlPos0
                    ;
                    mov     esi, edi
                    inc     esi
                    ;
                    cld
                    rep     movsb
                    mov     byte ptr [edi], ah
                    call    RCLSetLen
                    ;
                    popad
                    ret
                    ;
_RCLShlPos0:        mov     byte ptr [edi], ah
                    jz      short _RCLShlPos2
                    ;call    RCLDetNewLen
                    ;
                    ;popad
                    ;ret
                    ;
                    ; not full
                    ; --------
_RCLShlPos1:        cmp     edx, ecx
                    jae     short _eRCLShlPos
                    ;
                    add     edi, edx
                    sub     ecx, edx
                    
                    mov     esi, edi
                    inc     esi
                    ;
                    cld
                    rep     movsb
                    ;
                    dec     edi
                    call    RCLSetLen
                    ;
_RCLShlPos2:        call    RCLDetNewLen
                    ;
                    popad
                    ret
                    ;
_eRCLShlPos:        stc 
                    popad
                    ret
RCLShlPos           endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Shift right the string pointed by EBX from the position EDX
;
; Input:
;       ebx     - RCL string pointer
;       edx     - position (0,1,2,..., Max-1)
;       ah      - background char  
;
; Info:
;       clc:    string shifted or edx >=Len
;       stc:    Len = Max 
;
RCLShrPos           proc    
                    pushad
                    ;
                    call    RCLGetLen
                    jb      short _RCLShrPos1
                    ;
                    stc
                    popad
                    ret
                    ;
                    ; not full
                    ; --------
_RCLShrPos1:        cmp     edx, ecx 
                    jae     short _eRCLShrPos
                    ; 

                    add     edi, ecx
                    mov     esi, edi
                    inc     edi
                    call    RCLSetLen
                    ;
                    mov     edi, esi
                    dec     esi
                    ;
                    sub     ecx, edx
                    ;
                    std
                    rep     movsb
                    mov     byte ptr [edi], ah
                    cld
_eRCLShrPos:        clc
                    ;
                    popad
                    ret
RCLShrPos           endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Fills the string pointed by EBX with byte AL
;
; Input:
;       AL      - byte
;       EBX     - RCL string pointer
;
; Info:
;
RCLFilBytAL         proc    
                    pushad
                    ;
                    mov     ecx, dword ptr [ebx - $RclMax$]        ; Max
                    mov     dword ptr [ebx - $RclLen$], ecx        ; Len
                    xor     edx, edx
                    ;
@@:                 mov     byte ptr [ebx + edx], al
                    inc     edx
                    loop    short @B
                    ;
                    popad
                    ret
RCLFilBytAL         endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Copys string RCLStr1 to string RCLStr2 
;
; Input:
;       pRCLStr1 - string pointer 
;       pRCLStr2 - string pointer to where copy
;
; Info:
;
RCLCpyRCL           proc    pRCLStr1:DWORD, pRCLStr2:DWORD
                    pushad
                    ;
                    mov     ebx, pRCLStr2
                    mov     edi, ebx
                    mov     edx, dword ptr [ebx - $RclMax$]        ; Max Str DST
                    ;
                    mov     esi, pRCLStr1
                    mov     ecx, dword ptr [esi - $RclLen$]        ; Len Str FNT
                    ;
                    or      ecx, ecx
                    jz      short _RCLCpyRCL1
                    ;
                    cmp     ecx, edx
                    jbe     short _RCLCpyRCL0
                    ;
                    mov     ecx, edx
                    ;
_RCLCpyRCL0:        cld
                    rep     movsb
                    ;
_RCLCpyRCL1:        call    RCLSetLen
                    ;
                    popad
                    ret
RCLCpyRCL           endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Cleans the string pointed by EBX with spaces
;
; Input:
;        EBX     - RCL string pointer
;
RCLClean            proc    
                    mov     al, byte ptr _CleanChar
                    call    RCLFilBytAL
                    ret
RCLClean            endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Determine the new length of the string pointed by EBX
;
; Input:
;       ebx     - RCL string pointer
;
;
RCLDetNewLen        proc    
                    pushad
                    ;
                    mov     edi, ebx
                    mov     ecx, dword ptr [ebx - $RclLen$]        ; Len
                    cmp     ecx, 0
                    je      short _eRCLDetNewLen
                    ;
                    add     edi, ecx
                    dec     edi
                    ;
                    mov     al, byte ptr _CleanChar
                    std
                    repe    scasb
                    je      short @F
                    ;
                    inc     edi
                    ;    
@@:                 inc     edi
                    call    RCLSetLen
                    cld
                    ;
_eRCLDetNewLen:     clc
                    popad
                    ret
RCLDetNewLen        endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Gets the length of the string pointed by EBX
;
; Input:
;       EBX     - RCL string pointer
;
; Output:
;       EDI     - RCL string pointer
;       ECX     - Length 
;
RCLGetLen           proc
                    mov     edi, ebx
                    mov     ecx, dword ptr [ebx - $RclLen$]        ; Len
                    cmp     ecx, dword ptr [ebx - $RclMax$]        ; Max
                    ret
RCLGetLen           endp
; ---------------------------------------------------------------------------------------------
; Action:
;        Sets the length of the RCL string pointed by EBX
;
; Input:
;       EBX     - RCL string pointer
;       EDI     - position after the last byte in the string
;
; Output:
;       clc
;
RCLSetLen           proc
                    push    edi
                    ;
                    sub     edi, ebx
                    mov     dword ptr [ebx - $RclLen$], edi
                    clc
                    ;
                    pop     edi
                    ret
RCLSetLen           endp
; 

